﻿using System.Diagnostics;

namespace TestLibrary
{
    /// <summary>
    /// 윈도우즈 서비스 헬퍼
    /// </summary>
    public static class WindowsServiceHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public
 
        #region 윈도우즈 서비스 생성하기 - CreateWindowsService(serviceName, argumentList)
 
        /// <summary>
        /// 윈도우즈 서비스 생성하기
        /// </summary>
        /// <param name="serviceName">서비스명</param>
        /// <param name="argumentList">인자 리스트</param>
        /// <returns>처리 결과</returns>
        public static int CreateWindowsService(string serviceName, string argumentList)
        {
            Process process = GetProcess("sc.exe", $"create {serviceName} binPath= \"{argumentList}\"");
 
            process.Start();
 
            process.WaitForExit();
 
            return process.ExitCode;
        }
 
        #endregion
        #region 윈도우즈 서비스 시작하기 - StartWindowsService(serviceName)
 
        /// <summary>
        /// 윈도우즈 서비스 시작하기
        /// </summary>
        /// <param name="serviceName">서비스명</param>
        /// <returns>처리 결과</returns>
        public static int StartWindowsService(string serviceName)
        {
            Process process = GetProcess("sc.exe", $"start {serviceName}");
 
            process.Start();
 
            process.WaitForExit();
 
            return process.ExitCode;
        }
 
        #endregion
        #region 윈도우즈 서비스 중단하기 - StopWindowsService(serviceName)
 
        /// <summary>
        /// 윈도우즈 서비스 중단하기
        /// </summary>
        /// <param name="serviceName">서비스명</param>
        /// <returns>처리 결과</returns>
        public static int StopWindowsService(string serviceName)
        {
            Process process = GetProcess("sc.exe", $"stop {serviceName}");
 
            process.Start();
 
            process.WaitForExit();
 
            return process.ExitCode;
        }
 
        #endregion
        #region 윈도우즈 서비스 제거하기 - DeleteWindowsService(serviceName)
 
        /// <summary>
        /// 윈도우즈 서비스 제거하기
        /// </summary>
        /// <param name="serviceName">서비스명</param>
        /// <returns>처리 결과</returns>
        public static int DeleteWindowsService(string serviceName)
        {
            Process process = GetProcess("sc.exe", $"delete {serviceName}");
 
            process.Start();
 
            process.WaitForExit();
 
            return process.ExitCode;
        }
 
        #endregion
 
        //////////////////////////////////////////////////////////////////////////////// Private
 
        #region 프로세스 구하기 - GetProcess(filePath, argumentList)
 
        /// <summary>
        /// 프로세스 구하기
        /// </summary>
        /// <param name="filePath">파일 경로</param>
        /// <param name="argumentList">인자 리스트</param>
        /// <returns>프로세스</returns>
        private static Process GetProcess(string filePath, string argumentList)
        {
            Process process = new Process();
 
            process.StartInfo.UseShellExecute = true;
            process.StartInfo.FileName        = filePath;
            process.StartInfo.WindowStyle     = ProcessWindowStyle.Hidden;
            process.StartInfo.Verb            = "runas";
            process.StartInfo.Arguments       = argumentList;
 
            return process;
        }
 
        #endregion
    }
}