﻿using System;
using System.Windows.Forms;

namespace TestTray
{
    /// <summary>
    /// 메인 애플리케이션 컨텍스트
    /// </summary>
    public class MainApplicationContext : ApplicationContext
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// Configuration 메뉴 항목
        /// </summary>
        private MenuItem configurationMenuItem;

        /// <summary>
        /// Exit 메뉴 항목
        /// </summary>
        private MenuItem exitMenuItem;

        /// <summary>
        /// 알림 아이콘
        /// </summary>
        private NotifyIcon notifyIcon;


        /// <summary>
        /// 구성 폼
        /// </summary>
        private ConfigurationForm configurationForm;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainApplicationContext()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainApplicationContext()
        {
            this.configurationMenuItem = new MenuItem();

            this.configurationMenuItem.Name = "configurationMenuItem";
            this.configurationMenuItem.Text = "Configuration";


            this.exitMenuItem = new MenuItem();

            this.exitMenuItem.Name = "exitMenuItem";
            this.exitMenuItem.Text = "Exit";


            this.notifyIcon = new NotifyIcon();

            this.notifyIcon.Icon        = TestTray.Properties.Resources.tray;
            this.notifyIcon.ContextMenu = new ContextMenu(new MenuItem[] { configurationMenuItem, exitMenuItem });
            this.notifyIcon.Visible     = true;


            this.configurationForm = new ConfigurationForm();


            this.configurationMenuItem.Click += configurationMenuItem_Click;
            this.exitMenuItem.Click          += exitMenuItem_Click;
            this.notifyIcon.DoubleClick      += notifyIcon_DoubleClick;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Configuration 메뉴 항목 클릭시 처리하기 - configurationMenuItem_Click(sender, e)

        /// <summary>
        /// Configuration 메뉴 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void configurationMenuItem_Click(object sender, EventArgs e)
        {
            if(this.configurationForm.Visible)
            {
                this.configurationForm.Focus();
            }
            else
            {
                this.configurationForm.ShowDialog();
            }
        }

        #endregion
        #region Exit 메뉴 항목 클릭시 처리하기 - exitMenuItem_Click(sender, e)

        /// <summary>
        /// Exit 메뉴 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void exitMenuItem_Click(object sender, EventArgs e)
        {
            this.notifyIcon.Visible = false;

            Application.Exit();
        }

        #endregion
        #region 알림 아이콘 더블 클릭시 처리하기 - notifyIcon_DoubleClick(sender, e)

        /// <summary>
        /// 알림 아이콘 더블 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void notifyIcon_DoubleClick(object sender, EventArgs e)
        {
            if(this.configurationForm.ShowMessage)
            {
                MessageBox.Show("Information", "Hello World", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        #endregion
    }
}